#![allow(non_snake_case)]

use std::ops::{Add, Sub, Mul, Div};

#[derive(Debug)]
pub struct Vector2D {
    pub X: f64,
    pub Y: f64,
}

pub trait VectorOperations {
    fn Magnitude(&self) -> f64;
    fn Normalize(&self) -> Vector2D;
    fn Dot(&self, vector: Vector2D) -> f64;
}

impl Vector2D {
    pub fn New(&self, nx: f64, ny: f64) -> Vector2D {
        Vector2D { X: nx, Y: ny }
    }

    pub fn SelfNormalize(&mut self) {
        let magnitude = self.Magnitude();
        self.X = self.X / magnitude;
        self.Y = self.Y / magnitude;
    }
}

impl VectorOperations for Vector2D {
    fn Magnitude(&self) -> f64 {
        (((self.X * self.X) + (self.Y * self.Y))).sqrt()
    }

    fn Normalize(&self) -> Vector2D {
        let magnitude = self.Magnitude();
        Vector2D::New(self, self.X / magnitude, self.Y / magnitude)
    }

    fn Dot(&self, rhs: Vector2D) -> f64 {
        (self.X * rhs.X) + (self.Y * rhs.Y)
    }
}

impl Add for Vector2D {
    type Output = Vector2D;

    fn add(self, rhs: Vector2D) -> Vector2D {
        Vector2D::New(&self, self.X + rhs.X, self.Y + rhs.Y)
    }
}

impl Sub for Vector2D {
    type Output = Vector2D;

    fn sub(self, rhs: Vector2D) -> Vector2D {
        Vector2D::New(&self, self.X - rhs.X, self.Y - rhs.Y)
    }
}

impl Mul for Vector2D {
    type Output = Vector2D;

    fn mul(self, rhs: Vector2D) -> Vector2D {
        Vector2D::New(&self, self.X * rhs.X, self.Y * rhs.Y)
    }
}

impl Div for Vector2D {
    type Output = Vector2D;

    fn div(self, rhs: Vector2D) -> Vector2D {
        Vector2D::New(&self, self.X / rhs.X, self.Y / rhs.Y)
    }
}
