#![allow(non_snake_case)]

use std::ops::{Add, Sub, Mul, Div};

#[derive(Debug)]
pub struct Vector3D {
    pub X: f64,
    pub Y: f64,
    pub Z: f64,
}

pub trait VectorOperations {
    fn Magnitude(&self) -> f64;
    fn Normalize(&self) -> Vector3D;
    fn Dot(&self, vector: Vector3D) -> f64;
}

impl Vector3D {
    pub fn New(&self, nx: f64, ny: f64, nz: f64) -> Vector3D {
        Vector3D {
            X: nx,
            Y: ny,
            Z: nz,
        }
    }

    pub fn SelfNormalize(&mut self) {
        let magnitude = self.Magnitude();
        self.X = self.X / magnitude;
        self.Y = self.Y / magnitude;
        self.Z = self.Z / magnitude;
    }
}

impl VectorOperations for Vector3D {
    fn Magnitude(&self) -> f64 {
        (((self.X * self.X) + (self.Y * self.Y) + (self.Z * self.Z))).sqrt()
    }

    fn Normalize(&self) -> Vector3D {
        let magnitude = self.Magnitude();
        Vector3D::New(self,
                      self.X / magnitude,
                      self.Y / magnitude,
                      self.Z / magnitude)
    }

    fn Dot(&self, rhs: Vector3D) -> f64 {
        (self.X * rhs.X) + (self.Y * rhs.Y) + (self.Z * rhs.Z)
    }
}

impl Add for Vector3D {
    type Output = Vector3D;

    fn add(self, rhs: Vector3D) -> Vector3D {
        Vector3D::New(&self, self.X + rhs.X, self.Y + rhs.Y, self.Z + rhs.Z)
    }
}

impl Sub for Vector3D {
    type Output = Vector3D;

    fn sub(self, rhs: Vector3D) -> Vector3D {
        Vector3D::New(&self, self.X - rhs.X, self.Y - rhs.Y, self.Z - rhs.Z)
    }
}

impl Mul for Vector3D {
    type Output = Vector3D;

    fn mul(self, rhs: Vector3D) -> Vector3D {
        Vector3D::New(&self, self.X * rhs.X, self.Y * rhs.Y, self.Z * rhs.Z)
    }
}

impl Div for Vector3D {
    type Output = Vector3D;

    fn div(self, rhs: Vector3D) -> Vector3D {
        Vector3D::New(&self, self.X / rhs.X, self.Y / rhs.Y, self.Z / rhs.Z)
    }
}
